/* Metageek WiSPY interface 
 * Mike Kershaw/Dragorn <dragorn@kismetwireless.net>
 *
 * This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Extra thanks to Ryan Woodings @ Metageek for interface documentation
 */

#include <stdio.h>
#include <usb.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>

#include <gtk/gtk.h>

#include "config.h"

#include "wispy_gtk.h"
#include "wispy_container.h"
#include "wispy_gtk_planar.h"
#include "wispy_gtk_spectral.h"

#define GETTEXT_PACKAGE	"wispy_gtk"
#define LOCALEDIR		"/usr/share/locale/wispy_gtk"

void Wispy_Alert_Dialog(char *text) {
	GtkWidget *dialog, *okbutton, *label;

	label = gtk_label_new(text);
	dialog = gtk_dialog_new_with_buttons ("WiSPY", NULL,
										  GTK_DIALOG_MODAL, NULL);
	gtk_window_set_default_size (GTK_WINDOW (dialog), 300, 100);
	okbutton = gtk_dialog_add_button (GTK_DIALOG (dialog), 
									  GTK_STOCK_OK, GTK_RESPONSE_NONE);
	g_signal_connect_swapped (GTK_OBJECT (dialog), 
							  "response", G_CALLBACK (gtk_widget_destroy), 
							  GTK_OBJECT (dialog));
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_widget_grab_focus(okbutton);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
	gtk_widget_show_all(dialog);
}

typedef struct _wg_aux {
	wispy_device_registry *wdr;
	GtkWidget *main_vbox;
	GtkWidget *main_window;
} wg_aux;

static void add_planar(gpointer *data, gpointer *aux) {
	wg_aux *auxptr = (wg_aux *) data;
	GtkWidget *planar;

	planar = wispy_planar_new(auxptr->wdr);
	gtk_box_pack_end(GTK_BOX(auxptr->main_vbox), planar, TRUE, TRUE, 0);
	gtk_widget_show(planar);
}

static void add_spectral(gpointer *data, gpointer *aux) {
	wg_aux *auxptr = (wg_aux *) data;
	GtkWidget *spectral;

	spectral = wispy_spectral_new(auxptr->wdr);
	gtk_box_pack_end(GTK_BOX(auxptr->main_vbox), spectral, TRUE, TRUE, 0);
	gtk_widget_show(spectral);
}

static void add_temporal(gpointer *data, gpointer *aux) {
	wg_aux *auxptr = (wg_aux *) data;
	GtkWidget *siggraph;

	Wispy_Alert_Dialog("Coming soon");
}

static GtkItemFactoryEntry main_menu_items[] = {
	{ "/_WiSPY",			NULL,			NULL,	0, "<Branch>" },
	{ "/WiSPY/_Quit",		"<control>Q",	gtk_main_quit, 0, "<Item>" },
	{ "/_View",		"NULL",			NULL,	0, "<Branch>" },
	{ "/_View/Add Planar", NULL, add_planar, 0, "<Item>" },
	{ "/_View/Add Spectral", NULL, add_spectral, 0, "<Item>" },
	{ "/_View/Add Topographic", NULL, add_temporal, 0, "<Item>" },
};

static gint nmain_menu_items = 
	sizeof (main_menu_items) / sizeof (main_menu_items[0]);

int main(int argc, char *argv[]) {
	GtkWidget *window, *vbox, *devpane;
	wg_aux auxptr;

#if 0
	GtkWidget *menubar, *menu, *mn_wispy, *menuitem;
#endif

	GtkWidget *menubar;

	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;

	GtkWidget *planar;

	wispy_device_registry wdr;

	int x;

	setlocale(LC_ALL, "");
	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
	textdomain(GETTEXT_PACKAGE);

	gtk_init(&argc, &argv);

	/* init now so we can pass to wdr */
	devpane = gtk_hbox_new(FALSE, 0);
	wdr_init(&wdr, devpane);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	gtk_window_set_title(GTK_WINDOW(window), "WiSPY");

	gtk_window_set_default_size(GTK_WINDOW(window), 750, 450);

	g_signal_connect(G_OBJECT (window), "delete_event",
					 G_CALLBACK (gtk_main_quit), NULL);

	accel_group = gtk_accel_group_new();

	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>",
										accel_group);
	gtk_item_factory_create_items(item_factory, nmain_menu_items,
								  main_menu_items, &auxptr);
	gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
	menubar = gtk_item_factory_get_widget(item_factory, "<main>");

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	/* Make the device panel to go on the bottom to show the info for
	 * the devices...  devpane got built earlier */
	gtk_box_pack_end(GTK_BOX(vbox), devpane, FALSE, FALSE, 0);

	gtk_widget_show(vbox);
	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
	gtk_widget_show(menubar);

	auxptr.wdr = &wdr;
	auxptr.main_vbox = vbox;
	auxptr.main_window = window;

	planar = wispy_planar_new(&wdr);
	gtk_box_pack_end(GTK_BOX(vbox), planar, TRUE, TRUE, 0);
	gtk_widget_show(planar);

	gtk_widget_show(window);

	gtk_main();

	return 0;
}	

