/* Metageek WiSPY interface 
 * Mike Kershaw/Dragorn <dragorn@kismetwireless.net>
 *
 * This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Extra thanks to Ryan Woodings @ Metageek for interface documentation
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>

#include "wispy_gtk_planar.h"

static void wispy_planar_class_init(WispyPlanarClass *class);
static void wispy_planar_init(WispyPlanar *graph);
static void wispy_planar_destroy(GtkObject *object);

static gint wispy_planar_configure(GtkWidget *widget, 
									 GdkEventConfigure *event);
static gboolean wispy_planar_expose(GtkWidget *widget, 
									  GdkEventExpose *event,
									  gpointer *aux);
static gboolean wispy_planar_button_press(GtkWidget *widget,
											GdkEventButton *event,
											gpointer *aux);
static gboolean wispy_planar_mouse_move(GtkWidget *widget,
										GdkEventMotion *event,
										gpointer *aux);

G_DEFINE_TYPE(WispyPlanar, wispy_planar, WISPY_TYPE_WIDGET);

void wispy_planar_draw(GtkWidget *widget, cairo_t *cr, WispyWidget *wwidget) {
	cairo_text_extents_t extents;
	int x, chpix, maxcw;
	const double dash_onoff[] = {2, 4};
	const double dash_ononoff[] = {4, 2};

	char mtext[128];

	int chanmod;

	g_return_if_fail(widget != NULL);

	cairo_save(cr);

	/* Render the peak points */
	cairo_save(cr);
	cairo_new_path(cr);
	cairo_move_to(cr, wwidget->g_start_x + 0.5, wwidget->g_end_y - 0.5);
	for (x = 0; x < wwidget->sweepcache->avg->num_samples; x++) {
		int px, py;
		chpix = x * wwidget->wbar;

		px = wwidget->g_start_x + chpix;
		py = (float) wwidget->g_len_y * 
			(float) ((float) (abs(wwidget->sweepcache->peak->sample_data[x]) + 
							  wwidget->base_db_offset) /
					 (float) (abs(wwidget->sweepcache->avg->min_sample) + 
							  wwidget->base_db_offset));

		if (px < wwidget->g_start_x)
			px = wwidget->g_start_x;
		if (px > wwidget->g_end_x)
			px = wwidget->g_end_x;

		if (py < wwidget->g_start_y)
			py = wwidget->g_start_y;
		if (py > wwidget->g_end_y)
			py = wwidget->g_end_y;

		cairo_line_to(cr, px + 0.5, py + 0.5);
	}
	/* Close the path along the bottom */
	cairo_line_to(cr, wwidget->g_end_x - 0.5, wwidget->g_end_y - 0.5);
	cairo_line_to(cr, wwidget->g_start_x + 0.5, wwidget->g_end_y - 0.5);
	cairo_close_path(cr);
	/* Plot it - save the path so we can stroke and fill */
	cairo_set_source_rgb(cr, 0, 0, HC2CC(0xAA));
	cairo_fill_preserve(cr);
	cairo_set_source_rgb(cr, 1, 1, 1);
	cairo_set_line_width(cr, 1);
	cairo_stroke(cr);

	cairo_restore(cr);

	/* Render the average points */
	cairo_save(cr);
	cairo_new_path(cr);
	cairo_move_to(cr, wwidget->g_start_x + 0.5, wwidget->g_end_y - 0.5);
	for (x = 0; x < wwidget->sweepcache->avg->num_samples; x++) {
		int px, py;
		chpix = x * wwidget->wbar;

		px = wwidget->g_start_x + chpix;
		py = (float) wwidget->g_len_y * 
			(float) ((float) (abs(wwidget->sweepcache->avg->sample_data[x]) + 
							  wwidget->base_db_offset) /
					 (float) (abs(wwidget->sweepcache->avg->min_sample) + 
							  wwidget->base_db_offset));
		if (px < wwidget->g_start_x)
			px = wwidget->g_start_x;
		if (px > wwidget->g_end_x)
			px = wwidget->g_end_x;

		if (py < wwidget->g_start_y)
			py = wwidget->g_start_y;
		if (py > wwidget->g_end_y)
			py = wwidget->g_end_y;

		cairo_line_to(cr, px + 0.5, py + 0.5);
	}
	/* Close the path along the bottom */
	cairo_line_to(cr, wwidget->g_end_x - 0.5, wwidget->g_end_y - 0.5);
	cairo_line_to(cr, wwidget->g_start_x + 0.5, wwidget->g_end_y - 0.5);
	cairo_close_path(cr);
	/* Plot it */
	cairo_set_source_rgb(cr, HC2CC(0x11), HC2CC(0xBB), HC2CC(0x11));
	cairo_fill_preserve(cr);
	cairo_set_source_rgb(cr, 1, 1, 1);
	cairo_set_line_width(cr, 1);
	cairo_stroke(cr);
	cairo_restore(cr);

	/* Render the latest points */
	cairo_save(cr);
	cairo_new_path(cr);
	cairo_move_to(cr, wwidget->g_start_x + 0.5, wwidget->g_end_y - 0.5);
	for (x = 0; x < wwidget->sweepcache->avg->num_samples; x++) {
		int px, py;
		chpix = x * wwidget->wbar;

		px = wwidget->g_start_x + chpix;
		py = (float) wwidget->g_len_y * 
			(float) ((float) (abs(wwidget->sweepcache->sweeplist[
								  wwidget->sweepcache->pos]->sample_data[x]) + 
							  wwidget->base_db_offset) /
					 (float) (abs(wwidget->sweepcache->avg->min_sample) + 
							  wwidget->base_db_offset));

		if (px < wwidget->g_start_x)
			px = wwidget->g_start_x;
		if (px > wwidget->g_end_x)
			px = wwidget->g_end_x;

		if (py < wwidget->g_start_y)
			py = wwidget->g_start_y;
		if (py > wwidget->g_end_y)
			py = wwidget->g_end_y;

		cairo_line_to(cr, px + 0.5, py + 0.5);
	}
	cairo_line_to(cr, wwidget->g_end_x - 0.5, wwidget->g_end_y - 0.5);
	cairo_close_path(cr);
	/* Plot it */
	cairo_set_line_width(cr, 2);
	cairo_set_source_rgb(cr, HC2CC(0xFF), HC2CC(0xFF), HC2CC(0x00));
	cairo_stroke(cr);
	cairo_restore(cr);

	cairo_restore(cr);
}

static void wispy_planar_wdr_devopen(int slot, void *aux) {
	WispyPlanar *planar;
	WispyWidget *wwidget;

	g_return_if_fail(aux != NULL);
	g_return_if_fail(IS_WISPY_PLANAR(aux));
	g_return_if_fail(IS_WISPY_WIDGET(aux));

	planar = WISPY_PLANAR(aux);
	wwidget = WISPY_WIDGET(aux);
}

static void wispy_planar_wdr_sweep(int slot, int mode,
								   wispy_sample_sweep *sweep, void *aux) {
	WispyPlanar *planar;
	WispyWidget *wwidget;

	g_return_if_fail(aux != NULL);
	g_return_if_fail(IS_WISPY_PLANAR(aux));
	g_return_if_fail(IS_WISPY_WIDGET(aux));

	planar = WISPY_PLANAR(aux);
	wwidget = WISPY_WIDGET(aux);
}

static gint wispy_planar_button_press(GtkWidget *widget, 
									  GdkEventButton *event,
									  gpointer *aux) {
	WispyPlanar *planar;
	WispyWidget *wwidget;
	int ch;

	g_return_if_fail(aux != NULL);
	g_return_if_fail(IS_WISPY_PLANAR(aux));
	g_return_if_fail(IS_WISPY_WIDGET(aux));

	planar = WISPY_PLANAR(aux);
	wwidget = WISPY_WIDGET(aux);

	g_return_if_fail(wwidget->chanset != NULL);
	g_return_if_fail(wwidget->sweepcache != NULL);
	g_return_if_fail(wwidget->sweepcache->avg != NULL);

	if (event->button != 1)
		return TRUE;

	if ((ch = wispy_widget_find_chan_pt(wwidget, event->x, event->y)) > -1) {
		/* Should never get here if we don't have a chanset, do some
		 * checking anyhow though */
		if (ch < 0 || ch > wwidget->chanset->chan_num)
			return TRUE;

		if (wwidget->chanhit[ch]) {
			wwidget->chanhit[ch] = 0;
		} else {
			wwidget->chanhit[ch] = 1;
		}

		wwidget->hi_chan = -1;

	}

	wispy_widget_update(GTK_WIDGET(wwidget));

	return TRUE;
}

static gboolean wispy_planar_mouse_move(GtkWidget *widget,
										GdkEventMotion *event,
										gpointer *aux) {
	int x, y;
	int ch;
	GdkModifierType state;
	WispyPlanar *planar;
	WispyWidget *wwidget;

	g_return_if_fail(aux != NULL);
	g_return_if_fail(IS_WISPY_PLANAR(aux));
	g_return_if_fail(IS_WISPY_WIDGET(aux));

	planar = WISPY_PLANAR(aux);
	wwidget = WISPY_WIDGET(aux);

	g_return_if_fail(wwidget->chanset != NULL);
	g_return_if_fail(wwidget->sweepcache != NULL);
	g_return_if_fail(wwidget->sweepcache->avg != NULL);

	if (event->is_hint) {
		gdk_window_get_pointer(event->window, &x, &y, &state);
	} else {
		x = (int) event->x;
		y = (int) event->y;
		state = event->state;
	}

	/* If the button is down and we have a current marker, move it around */
	if (x > wwidget->g_start_x && x < wwidget->g_end_x &&
		y > wwidget->g_start_y && y < wwidget->g_end_y &&
		planar->mouse_down && (state & GDK_BUTTON1_MASK)) {

	}

	/* Search for the channel positions, update the graph if we've changed
	 * the highlighted channel */
	if ((ch = wispy_widget_find_chan_pt(wwidget, x, y)) >= -1) {
		if (ch != wwidget->hi_chan) {
			wwidget->hi_chan = ch;
			wispy_widget_update(GTK_WIDGET(wwidget));
		}
	} else if (wwidget->hi_chan > -1) {
		wwidget->hi_chan = -1;
		wispy_widget_update(GTK_WIDGET(wwidget));
	}

	return TRUE;
}

static void wispy_planar_class_init(WispyPlanarClass *class) {
	GObjectClass *gobject_class;
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	gobject_class = G_OBJECT_CLASS(class);
	object_class = GTK_OBJECT_CLASS(class);
	widget_class = GTK_WIDGET_CLASS(class);

	object_class->destroy = wispy_planar_destroy;
}

static void wispy_planar_destroy(GtkObject *object) {
	WispyPlanar *planar = WISPY_PLANAR(object);
	WispyWidget *wwidget;

	wwidget = WISPY_WIDGET(planar);

	GTK_OBJECT_CLASS(wispy_planar_parent_class)->destroy(object);
}

GtkWidget *wispy_planar_new(wispy_device_registry *wdr) {
	WispyPlanar *planar;
	WispyWidget *wwidget;

	planar = gtk_type_new(wispy_planar_get_type());

	wwidget = WISPY_WIDGET(planar);

	wwidget->wdr = wdr;

	return GTK_WIDGET(planar);
}

wispy_planar_marker *wispy_planar_marker_new(double r, double g, double b,
											 int khz) {
	wispy_planar_marker *mkr = malloc(sizeof(wispy_planar_marker));
	cairo_t *cr;
	GdkPixmap *pixmap;

	mkr->r = r;
	mkr->g = g;
	mkr->b = b;
	mkr->khz = khz;

	mkr->cur = mkr->avg = mkr->peak = 0;

	pixmap = gdk_pixmap_new(NULL, 9, 9, 24);
	cr = gdk_cairo_create(pixmap);
	cairo_rectangle(cr, 0, 0, 9, 9);
	cairo_clip(cr);
	cairo_set_source_rgb(cr, r, g, b);
	cairo_rectangle(cr, 0, 0, 9, 9);
	cairo_fill_preserve(cr);
	cairo_stroke(cr);
	cairo_destroy(cr);

	/* mkr->pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 9, 9); */
	mkr->pixbuf = gdk_pixbuf_get_from_drawable(NULL, pixmap, 
											   gdk_drawable_get_colormap(pixmap),
											   0, 0, 0, 0, 9, 9);

	return mkr;
}

gboolean wispy_planar_mkrlist_selected(GtkTreeSelection *selection,
									   GtkTreeModel *model,
									   GtkTreePath *path,
									   gboolean selected,
									   gpointer aux) {
	GtkTreeIter iter;
	WispyPlanar *planar = (WispyPlanar *) aux;

	if (gtk_tree_model_get_iter(model, &iter, path) && selected) {
		void *mptr;

		gtk_tree_model_get(model, &iter, 5, &mptr, -1);

		planar->cur_mkr = mptr;

		printf("debug - selected marker %p\n", planar->cur_mkr);
	} else {
		planar->cur_mkr = NULL;
	}

	return TRUE;
}

static void wispy_planar_init(WispyPlanar *planar) {
	WispyWidget *wwidget;
	GtkWidget *scrollwindow;
	
	GtkCellRenderer *cell;
	GtkTreeViewColumn *column;
	GtkTreeIter iter;
	GtkTreeSelection *selection;

	GtkWidget *clabel;
	PangoAttrList	*attr_lst;
	PangoAttribute	*attr;

	GtkWidget *bhb;

	GdkColor c;
	GtkStyle *style;

	wispy_planar_marker *mkr;

	wwidget = WISPY_WIDGET(planar);

	wwidget->sweep_num_samples = WISPY_PLANAR_NUM_SAMPLES;
	wwidget->sweep_keep_avg = 1;
	wwidget->sweep_keep_peak = 1;

	wwidget->sweep_num_aggregate = 2;

	wwidget->hlines = 8;
	wwidget->base_db_offset = 0;

	wwidget->graph_title = "<b>Planar View</b>";
	wwidget->graph_title_bg = "#00CC00";
	wwidget->graph_control_bg = "#A0CCA0";

	wwidget->show_channels = 1;
	wwidget->show_dbm = 1;
	wwidget->show_dbm_lines = 1;

	wwidget->wdr_sweep_func = wispy_planar_wdr_sweep;
	wwidget->wdr_devopen_func = wispy_planar_wdr_devopen;
	wwidget->draw_mouse_move_func = wispy_planar_mouse_move;
	wwidget->draw_mouse_click_func = wispy_planar_button_press;
	wwidget->draw_func = wispy_planar_draw;

	wispy_widget_buildgui(wwidget);

	planar->mkr_list = NULL;
	planar->cur_mkr = NULL;

	scrollwindow = gtk_scrolled_window_new(NULL, NULL);

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwindow),
								   GTK_POLICY_NEVER,
								   GTK_POLICY_AUTOMATIC);

	planar->mkr_treeview = gtk_tree_view_new();

	/* Force the treeview bg colors */
	gdk_color_parse(wwidget->graph_control_bg, &c);
	style = gtk_style_new();
	gtk_widget_set_style(GTK_WIDGET(planar->mkr_treeview), style);
	style->bg[GTK_STATE_NORMAL] = c;
	style->base[GTK_STATE_NORMAL] = c;
	gtk_style_unref(style);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollwindow),
										  planar->mkr_treeview);

	/* Relly now, this is the ONLY way to do all this? */
	attr_lst = pango_attr_list_new();
	attr = pango_attr_size_new(7.0 * PANGO_SCALE);
	attr->start_index = 0;
	attr->end_index = 100;
	pango_attr_list_insert(attr_lst, attr);

	cell = gtk_cell_renderer_pixbuf_new();

	column = gtk_tree_view_column_new();
	gtk_tree_view_column_pack_start(column, cell, FALSE);
	gtk_tree_view_column_add_attribute(column, cell, "pixbuf", 0);
	gtk_tree_view_append_column(GTK_TREE_VIEW(planar->mkr_treeview),
								GTK_TREE_VIEW_COLUMN(column));

	cell = gtk_cell_renderer_text_new();
	g_object_set(cell, "size-points", 6.0, NULL);
	g_object_set(cell, "xalign", 0.5, NULL);

	column = gtk_tree_view_column_new();
	gtk_tree_view_column_pack_start(column, cell, FALSE);
	gtk_tree_view_column_add_attribute(column, cell, "text", 1);
	clabel = gtk_label_new("Freq");
	gtk_label_set_use_markup(GTK_LABEL(clabel), FALSE);
	gtk_label_set_use_underline(GTK_LABEL(clabel), FALSE);
	gtk_label_set_attributes(GTK_LABEL(clabel), attr_lst);
	gtk_tree_view_column_set_widget(column, clabel);
	gtk_widget_show(clabel);
	gtk_tree_view_append_column(GTK_TREE_VIEW(planar->mkr_treeview),
								GTK_TREE_VIEW_COLUMN(column));
	
	column = gtk_tree_view_column_new();
	gtk_tree_view_column_pack_start(column, cell, FALSE);
	gtk_tree_view_column_add_attribute(column, cell, "text", 2);
	clabel = gtk_label_new("Cur");
	gtk_label_set_use_markup(GTK_LABEL(clabel), FALSE);
	gtk_label_set_use_underline(GTK_LABEL(clabel), FALSE);
	gtk_label_set_attributes(GTK_LABEL(clabel), attr_lst);
	gtk_tree_view_column_set_widget(column, clabel);
	gtk_widget_show(clabel);
	gtk_tree_view_append_column(GTK_TREE_VIEW(planar->mkr_treeview),
								GTK_TREE_VIEW_COLUMN(column));

	column = gtk_tree_view_column_new();
	gtk_tree_view_column_pack_start(column, cell, FALSE);
	gtk_tree_view_column_add_attribute(column, cell, "text", 3);
	clabel = gtk_label_new("Avg");
	gtk_label_set_use_markup(GTK_LABEL(clabel), FALSE);
	gtk_label_set_use_underline(GTK_LABEL(clabel), FALSE);
	gtk_label_set_attributes(GTK_LABEL(clabel), attr_lst);
	gtk_tree_view_column_set_widget(column, clabel);
	gtk_widget_show(clabel);
	gtk_tree_view_append_column(GTK_TREE_VIEW(planar->mkr_treeview),
								GTK_TREE_VIEW_COLUMN(column));

	column = gtk_tree_view_column_new();
	gtk_tree_view_column_pack_start(column, cell, TRUE);
	gtk_tree_view_column_add_attribute(column, cell, "text", 4);
	clabel = gtk_label_new("Max");
	gtk_label_set_use_markup(GTK_LABEL(clabel), FALSE);
	gtk_label_set_use_underline(GTK_LABEL(clabel), FALSE);
	gtk_label_set_attributes(GTK_LABEL(clabel), attr_lst);
	gtk_tree_view_column_set_widget(column, clabel);
	gtk_widget_show(clabel);
	gtk_tree_view_append_column(GTK_TREE_VIEW(planar->mkr_treeview),
								GTK_TREE_VIEW_COLUMN(column));

	gtk_box_pack_start(GTK_BOX(wwidget->vbox), scrollwindow, TRUE, TRUE, 2);

	bhb = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(wwidget->vbox), bhb, FALSE, FALSE, 0);
	planar->mkr_newbutton = gtk_button_new_with_label("Add");
	gtk_box_pack_start(GTK_BOX(bhb), planar->mkr_newbutton, TRUE, TRUE, 0);
	planar->mkr_delbutton = gtk_button_new_with_label("Del");
	gtk_box_pack_start(GTK_BOX(bhb), planar->mkr_delbutton, TRUE, TRUE, 0);
	gtk_widget_show(planar->mkr_newbutton);
	gtk_widget_show(planar->mkr_delbutton);
	gtk_widget_set_sensitive(planar->mkr_newbutton, 0);
	gtk_widget_set_sensitive(planar->mkr_delbutton, 0);
	gtk_widget_show(bhb);

	planar->mkr_treelist = gtk_list_store_new(6, GDK_TYPE_PIXBUF, G_TYPE_STRING, 
											  G_TYPE_STRING, G_TYPE_STRING, 
											  G_TYPE_STRING, G_TYPE_POINTER);
	gtk_tree_view_set_model(GTK_TREE_VIEW(planar->mkr_treeview),
							GTK_TREE_MODEL(planar->mkr_treelist));

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(planar->mkr_treeview));
	gtk_tree_selection_set_select_function(selection, wispy_planar_mkrlist_selected, 
										   planar, NULL);

	/* Auto delete when we don't ref it anymore */
	g_object_unref(planar->mkr_treelist);
	pango_attr_list_unref(attr_lst);

	mkr = wispy_planar_marker_new(1, 0, 0, 0);
	planar->mkr_list = g_list_append(planar->mkr_list, mkr);
	gtk_list_store_append(GTK_LIST_STORE(planar->mkr_treelist), &iter);
	gtk_list_store_set(GTK_LIST_STORE(planar->mkr_treelist), &iter,
					   0, mkr->pixbuf,
					   1, "0",
					   2, "0",
					   3, "0",
					   4, "0",
					   5, mkr,
					   -1);
	/* Select the first one */
	gtk_tree_selection_unselect_all(selection);
	gtk_tree_selection_select_iter(selection, &iter);

	mkr = wispy_planar_marker_new(0, 1, 0, 0);
	planar->mkr_list = g_list_append(planar->mkr_list, mkr);
	gtk_list_store_append(GTK_LIST_STORE(planar->mkr_treelist), &iter);
	gtk_list_store_set(GTK_LIST_STORE(planar->mkr_treelist), &iter,
					   0, mkr->pixbuf,
					   1, "0",
					   2, "0",
					   3, "0",
					   4, "0",
					   5, mkr,
					   -1);

	mkr = wispy_planar_marker_new(0, 0, 1, 0);
	planar->mkr_list = g_list_append(planar->mkr_list, mkr);
	gtk_list_store_append(GTK_LIST_STORE(planar->mkr_treelist), &iter);
	gtk_list_store_set(GTK_LIST_STORE(planar->mkr_treelist), &iter,
					   0, mkr->pixbuf,
					   1, "0",
					   2, "0",
					   3, "0",
					   4, "0",
					   5, mkr,
					   -1);

	mkr = wispy_planar_marker_new(1, 1, 0, 0);
	planar->mkr_list = g_list_append(planar->mkr_list, mkr);
	gtk_list_store_append(GTK_LIST_STORE(planar->mkr_treelist), &iter);
	gtk_list_store_set(GTK_LIST_STORE(planar->mkr_treelist), &iter,
					   0, mkr->pixbuf,
					   1, "0",
					   2, "0",
					   3, "0",
					   4, "0",
					   5, mkr,
					   -1);

	gtk_widget_show(planar->mkr_treeview);
	gtk_widget_show(scrollwindow);

}

